%% Pseudorandom brain data IRGN-T(G)V [1] reconstruction:
% 12 channel brain data compressed to 6 virtual channels.
% The pseudorandom sampling pattern was generated according to [2].
%
% Some experiments from [1] can be reproduced with this code. To 
% switch between L2, TV and TGV regularization, just change the value of 
% the variable params.tvtype. In the paper, two different regularization
% parameters were used. In the case of moderate undersampling, the final 
% regularization parameter is reduced to zero. A more pronounced T(G)V
% filtering effect in the case of high undersampling can be achieved by
% stopping this decrease during the iterations. This can be controlled by
% setting the variable params.beta_min either to 0 or to, e.g., 5e-3.
%
% [1] Knoll, F.; Clason, C.; Bredies, K.; Uecker, M.; Stollberger R.:
% Parallel imaging with nonlinear reconstruction using variational 
% penalties.
% Magnetic Resonance in Medicine, Early View: DOI: 10.1002/mrm.22964.
% 
% [2] Knoll, F.; Clason, C.; Diwoky, C.; Stollberger, R.:
% Adapted random sampling patterns for accelerated MRI.
% Magnetic Resonance Materials in Physics, Biology and Medicine 24:43-50
% (2011).

% Christian Clason and Florian Knoll, July 2011
% christian.clason@uni-graz.at
% http://www.uni-graz.at/~clason 
% florian.knoll@tugraz.at
% http://www.imt.tugraz.at/research/mr-image-reconstruction

clear all; close all; clc;

%% Reconstruction parameters
params.maxit    = 6; % maximum number of IRGN iterations
params.tvtype   = 1; % regularization term: 0: L2, 1: TV, 2: TGV
params.beta_min = 0; % final value of beta: 0: no T(G)V effect, >0 effect

%% Load data
load random_brain_data;
[nPE1,nPE2,nCh] = size(rawdata);

%% Generate sampling operator
pattern = abs(rawdata(:,:,1))>0;
ft  = @(x) pattern.*fftshift(fft2(fftshift(x)))./sqrt(nPE1.*nPE2);
ift = @(x) fftshift(ifft2(fftshift(pattern.*x))).*sqrt(nPE1.*nPE2);

%% SOS image
img_sos=zeros(nPE1,nPE2,nCh);
for ii = 1:nCh
    img_sos(:,:,ii) = ift(rawdata(:,:,ii));
end
img_sos = sqrt(sum(abs(img_sos).^2,3));
figure, imshow(rot90(img_sos',2),[]); title('SOS without DensComp');

%% IRGN-T(G)V reconstruction
% usually there should be no need to change these
params.alpha0    = 1;      % initial penalty alpha_0 (L2, sensitivites)
params.beta0     = 1;      % initial penalty beta_0 (image)
params.alpha_min = 0;      % final value of alpha
params.alpha_q   = 1/10;   % reduction factor for alpha
params.beta_q 	 = 1/5;    % reduction factor for beta 
params.tvits     = 20;     % initial number of gradient steps

tic
disp('-------------------------');
disp('Starting IRGN-T(G)V');
disp('-------------------------');
imgIRGNTV = irgn_solve(rawdata,ft,ift,params);
disp(['Time: ', num2str(toc/60), ' min']);

%% Display results
figure, imshow(abs(rot90(imgIRGNTV',2)),[]); title('IRGN');
